@@ -5,8 +5,7 @@ wrapper around PyOpenCL and py2opencl Python -> OpenCL conversion utility
 import pyopencl as cl
 import numpy as np
 
-from .convert import lambda_to_kernel
-
+from .convert import lambda_to_kernel, nptyp_to_cl, cltyp_to_np
 
 import os
 os.environ['PYOPENCL_COMPILER_OUTPUT']='1'
@@ -41,13 +40,7 @@ class Py2OpenCL(object):
         length, types = None, []
         for a in arrays:
             try:
-                types.append( {np.dtype('float16'): 'half',
-                               np.dtype('float32'): 'float',
-                               np.dtype('float64'): 'double',
-                               np.dtype('uint8'): 'uchar',
-                               np.dtype('int16'): 'short',
-                               np.dtype('int32'): 'int',
-                               np.dtype('int64'): 'long'}[ a.dtype ] )
+                types.append( a.dtype )
             except KeyError:
                 raise ValueError("invalid numpy type: "+str(a.dtype))
 
@@ -57,25 +50,27 @@ class Py2OpenCL(object):
                 # FIXME: this precludes legitimate use-cases ...
                 assert len(a) == length
 
-        self.argnames, self._kernel = lambda_to_kernel( self.lmb, types, bindings=self.bindings )
+        self.argnames, self._kernel, cl_return_typ = lambda_to_kernel( self.lmb, types, bindings=self.bindings )
+        return_typ = cltyp_to_np[cl_return_typ]
+
         assert self.argnames and len(self.argnames) == len(arrays)
 
         # compile openCL
         self.prog = cl.Program(self.ctx, self._kernel).build()
 
         mf = cl.mem_flags
-        buffs, nbytes = [], arrays[0].nbytes
-        for arr in arrays:
-            buffs.append( cl.Buffer(self.ctx, mf.READ_ONLY | mf.COPY_HOST_PTR, hostbuf=arr ))
+        res_np = np.zeros( len(arrays[0]), dtype=return_typ )
+
+        buffs = [ cl.Buffer(self.ctx, mf.READ_ONLY | mf.COPY_HOST_PTR, hostbuf=arr )
+                  for arr in arrays ]
 
         # results:
-        buffs.append( cl.Buffer(self.ctx, mf.WRITE_ONLY, nbytes) )
+        buffs.append( cl.Buffer(self.ctx, mf.WRITE_ONLY, res_np.nbytes) )
 
         # run!
         self.prog.sum(self.queue, arrays[0].shape, None, *buffs)
 
-        res_np = np.empty_like(arrays[0])
-        cl.enqueue_copy(self.queue, res_np, buffs[-1])
+        cl.enqueue_copy( self.queue, res_np, buffs[-1] )
         return res_np.copy()
 
 
